<?php
/**
* © Copyright 2010 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package I2CE
* @subpackage I2CE
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.0.4
* @since v4.0.4
* @filesource 
*/ 
/** 
* Class I2CE_PrintedForm_Render
* 
* @access public
*/


class I2CE_PrintedForm_Render_ODT extends I2CE_PrintedForm_Render{

    protected function processElement_text($left_x,$top_y,$formData,$textProps, $elementConfig) {
        
    }

    protected function processElement_image($left_x,$top_y,$formData,$textProps, $elementConfig) {

    }

    protected function addPage($textProps)  {

    }


    /**
     *Abstract method to render the form. Makes sure all ducks are in a row
     * @returns boolean true on sucess.
     */
    public function render() {
        if (!is_string($this->std_form) || strlen($this->std_form) == 0) {
            I2CE::raiseError("No standard printed form set");
            return false;
        }
        $this->stdConfig = I2CE::getConfig()->traverse( '/modules/PrintedForms/forms/' . $this->std_form, false);
        if (!$this->stdConfig instanceof I2CE_MagicDataNode) {
            I2CE::raiseError("No standard printed form   /modules/PrintedForms/forms/" . $this->std_form);;
            return false;
        }
        if (!  $this->stdConfig->setIfIsSet($relationship, 'relationship' )) {
            I2CE::raiseError("No relationship set");
            return false;
        }
        try {
            $this->rel = new I2CE_FormRelationship($relationship, $this->base_rel_config);
        } catch (Exception $e) {
            I2CE::raiseError("Could not instatiate relationship $relationship");
            return false;
        } 
        $template = false;
        if (!  $this->stdConfig->setIfIsSet($template, 'template' )) {
            I2CE::raiseError("No template  set");
            return false;
        }
        $this->template_file = I2CE::getFileSearch()->search('ODT_TEMPLATES',$template);
        if (!$this->template_file) {
            I2CE::raiseError("No template file found from $template");
            return false;
        }
        $this->template_contents = new ZipArchive();
        if ($this->template_contents->open($this->template_file)!==TRUE) {
            I2CE::raiseError("Could not extract odt file");
            return;
        }
        if ( ($this->mimetype = $this->template_contents->getFromName('mimetype')) === false) {
            I2CE::raiseError("Could not determine mime type");
        } else {
            I2CE::raiseError("mime type " . $this->mimetype);
        }
        $this->content = $this->stdConfig->getAsArray('content');
        $forms = array();
        foreach ($this->ids as $id) {
            if (!is_string($id)) {
                continue;
            }
            $fs = $this->rel->getFormsSatisfyingRelationship($id);
            if (!is_array($fs) || count($fs) == 0) {
                continue;
            }
            $forms[$id] = $fs;
        }
        if (count($forms) == 0) {
            I2CE::raiseError("No valid forms");
            return false;
        }

        $this->forms = $forms;
        $textProps = array();
        I2CE::longExecution(  );
        $success = $this->_render($textProps);
        $this->template_contents->close();
        return $success;
    }

    /**
     * @var protected string $template_file. The absolute file location of the template we are reading from
     */
    protected $template_file = false;

    /**
     * @var protected string $template_contnets. The contents of the template file
     */
    protected $template_contents = false;

    /**
     * @var protected array $output. Indedx by id's of the output files
     */
    protected $output = array();

    protected $formData = array();

    /**
     *business  method to render the forms
     * @param array $textProps
     * @returns boolean true on sucess.
     */
    protected function _render($textProps) {
        $this->output = array();
        $counter = 0;
        foreach ($this->forms as $id=>$formData) {
            $this->setCurrentId($id);
            $this->formData = $formData;
            if (! ($file = $this->processFile())) {
                I2CE::raiseError("Could not add form: $id");
                return false;
            }
            $this->output[$id] = $file;
            $counter++;
        }
        return true;        
    }



    protected function processFile() {
        $temp = tempnam('/tmp', 'zip');                
        $zip = new ZipArchive(); 
        if (($zip->open($temp,ZipArchive::CREATE | ZipArchive::OVERWRITE)) === false) {
            I2CE::raiseError("Could not create $temp");
            return false;
        }

        for ($i=0; $i<$this->template_contents->numFiles;$i++) {
            $stats = $this->template_contents->statIndex($i);
            $contents = $this->template_contents->getFromIndex($i);
            if ($stats['name'] == 'content.xml') {
                $contents =$this->processContents($contents);
            }
            $zip->addFromString($stats['name'],$contents);
        }
        $zip->close();
        return file_get_contents($temp);
    }

    protected $user;

    protected function replacement($matches) {
        $replacement = $matches[0]; //default to the whole string i.e. we are not reaplcing.
        $original = $matches[1];
        if (substr($original,0,2) == '++') {
            //it is a special variable
            $spec = strtolower(substr($original,2,4));
            $arg = '';
            if (preg_match('/^\(\s*(.*?)\s*\)$/', substr($original,6), $matches)) {
                $arg = $matches[1];
            }
            switch ($spec) {
            case 'date':
                if (strlen($arg) == 0) {
                    $arg = '%x';
                }
                $replacement = @strftime($arg);
                break;
            case 'user':
                if (!$this->user instanceof I2CE_User) {
                    $this->user = new I2CE_User();
                }
                $replacement = $user->firstname . ' ' . $user->lastname;
                break;
            case 'eval':
                @eval('$replacement = ' . $arg . ';');
                break;
            default:
                //do nothing
                break;

            }
        } else if (substr($original,0,1) == '+') {
            //it is a relationship function
            $replacement  = $this->rel->evaluateFunction(substr($original,1),$this->formData);
        } else {
            //it is a form+field
            list($namedform,$field) = array_pad(explode("+",$original,2),2,'');
            $namedform = trim($namedform);
            $field = trim($field);
            if ($namedform == $this->stdConfig->relationship) {
                $namedform = 'primary_form';
            }            
            if ($namedform && $field && array_key_exists($namedform,$this->formData) && ($this->formData[$namedform] instanceof I2CE_Form)) {                
                $fieldObj = $this->formData[$namedform]->getField($field);
                if ($fieldObj instanceof I2CE_FormField) {
                    $replacement = $fieldObj->getDisplayValue();
                }
            }
        }
        return $replacement;
    }

    protected function processContents($contents) {
        return preg_replace_callback('/{{{([0-9a-zA-Z_\-\+]+)}}}/',array($this,'replacement'),$contents);
    }
    

    /**
     *Abstract method to retreive/display the contents of the rendered forms
     * @param boolean $as_string.  Defaults to false 
     * @returns mixed.  If {$as_string} is false the it is a  boolean true on sucess.  If $as_string is true, then it is a string on success, false on failure
     */
    public function display($as_string = false) {
        while (ob_get_level()  > 0) {
            if($cont = trim(ob_get_clean())) { 
                I2CE::raiseError("(".$cont .')');
            }
        }
        
        if (count($this->output) == 0) {
            if ($as_string) {
                return '';
            } else {
                return true;
            }
        } else if (count($this->output) == 1) {
            if ($as_string) {
                //we don't want mime type.  this goes to the archive
                reset($this->output);
                return current($this->output);
                exit;
            } else {
                //we do want mime type.  this goes to the browser
                reset($this->output);
                header($this->mimetype);
                $expires = 60*5; //5 minute expiration
                $expires =0;
                header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
                header('Expires: ' . gmdate('D, d M Y H:i:s', time()+$expires) . ' GMT');
                header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
                header('Content-Length: '.strlen(current($this->output)));
                header('Content-Type: ' . $this->mimetype);
                header('Content-disposition: attachment; filename="'. addslashes($this->getFileName()) .'"');
                echo current($this->output);
                exit; // we want to make sure there is no further output or that the $this->page->display() method is not called
            }
        } else {
            //do it as a zip file
            I2CE::raiseError("Zip archive not supported yet");
            return false;
        }
    }

    public  function getMimeType() {
        return $this->mimetype;
    }

    public  function getFileName() {
        return basename($this->template_file);
    }


    
  }
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
